import base64
import json
from datetime import datetime
from enum import Enum
from typing import List
from urllib.parse import urljoin

import requests

from pygrocy.utils import (
    localize_datetime,
    parse_bool_int,
    parse_date,
    parse_float,
    parse_int,
)

DEFAULT_PORT_NUMBER = 9192


class ShoppingListItem(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._product_id = parse_int(parsed_json.get("product_id", None))
        self._note = parsed_json.get("note", None)
        self._amount = parse_float(parsed_json.get("amount"), 0)
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp", None)
        )
        self._shopping_list_id = parse_int(parsed_json.get("shopping_list_id"))
        self._done = parse_int(parsed_json.get("done"))

    @property
    def id(self) -> int:
        return self._id

    @property
    def product_id(self) -> int:
        return self._product_id

    @property
    def note(self) -> str:
        return self._note

    @property
    def amount(self) -> float:
        return self._amount


class MealPlanResponse(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._day = parse_date(parsed_json.get("day"))
        self._type = parsed_json.get("type")
        self._recipe_id = parse_int(parsed_json.get("recipe_id"))
        self._recipe_servings = parse_int(parsed_json.get("recipe_servings"))
        self._note = parsed_json.get("note", None)
        self._product_id = parsed_json.get("product_id")
        self._product_amount = parse_float(parsed_json.get("product_amount"), 0)
        self._product_qu_id = parsed_json.get("product_qu_id")
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp")
        )
        self._userfields = parsed_json.get("userfields")

    @property
    def id(self) -> int:
        return self._id

    @property
    def day(self) -> datetime:
        return self._day

    @property
    def recipe_id(self) -> int:
        return self._recipe_id

    @property
    def recipe_servings(self) -> int:
        return self._recipe_servings

    @property
    def note(self) -> str:
        return self._note


class RecipeDetailsResponse(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._name = parsed_json.get("name")
        self._description = parsed_json.get("description")
        self._base_servings = parse_int(parsed_json.get("base_servings"))
        self._desired_servings = parse_int(parsed_json.get("desired_servings"))
        self._picture_file_name = parsed_json.get("picture_file_name")
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp")
        )
        self._userfields = parsed_json.get("userfields")

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def base_servings(self) -> int:
        return self._base_servings

    @property
    def desired_servings(self) -> int:
        return self._desired_servings

    @property
    def picture_file_name(self) -> str:
        return self._picture_file_name


class QuantityUnitData(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._name = parsed_json.get("name")
        self._name_plural = parsed_json.get("name_plural")
        self._description = parsed_json.get("description")
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp")
        )


class LocationData(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._name = parsed_json.get("name")
        self._description = parsed_json.get("description")
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp")
        )

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description


class ProductData(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))
        self._name = parsed_json.get("name")
        self._description = parsed_json.get("description", None)
        self._location_id = parse_int(parsed_json.get("location_id", None))
        self._product_group_id = parse_int(parsed_json.get("product_group_id", None))
        self._qu_id_stock = parse_int(parsed_json.get("qu_id_stock", None))
        self._qu_id_purchase = parse_int(parsed_json.get("qu_id_purchsase", None))
        self._qu_factor_purchase_to_stock = parse_float(
            parsed_json.get("qu_factor_purchase_to_stock", None)
        )
        self._picture_file_name = parsed_json.get("picture_file_name", None)
        self._allow_partial_units_in_stock = bool(
            parsed_json.get("allow_partial_units_in_stock", None) == "true"
        )
        self._row_created_timestamp = parse_date(
            parsed_json.get("row_created_timestamp", None)
        )
        self._min_stock_amount = parse_int(parsed_json.get("min_stock_amount", None), 0)
        self._default_best_before_days = parse_int(
            parsed_json.get("default_best_before_days", None)
        )

    @property
    def id(self) -> int:
        return self._id

    @property
    def product_group_id(self) -> int:
        return self._product_group_id

    @property
    def name(self) -> str:
        return self._name


class ChoreData(object):
    def __init__(self, parsed_json):
        self.id = parse_int(parsed_json.get("id"))
        self.name = parsed_json.get("name")
        self.description = parsed_json.get("description")
        self.period_type = parsed_json.get("period_type")
        self.period_config = parsed_json.get("period_config")
        self.period_days = parse_int(parsed_json.get("period_days"))
        self.track_date_only = parse_bool_int(parsed_json.get("track_date_only"))
        self.rollover = parse_bool_int(parsed_json.get("rollover"))
        self.assignment_type = parsed_json.get("assignment_type")
        self.assignment_config = parsed_json.get("assignment_config")
        self.next_execution_assigned_to_user_id = parse_int(
            parsed_json.get("next_execution_assigned_to_user_id")
        )
        self.userfields = parsed_json.get("userfields")


class UserDto(object):
    def __init__(self, parsed_json):
        self._id = parse_int(parsed_json.get("id"))

        self._username = parsed_json.get("username")
        self._first_name = parsed_json.get("first_name")
        self._last_name = parsed_json.get("last_name")
        self._display_name = parsed_json.get("display_name")

    @property
    def id(self) -> int:
        return self._id

    @property
    def username(self) -> str:
        return self._username

    @property
    def first_name(self) -> str:
        return self._first_name

    @property
    def last_name(self) -> str:
        return self._last_name

    @property
    def display_name(self) -> str:
        return self._display_name


class CurrentChoreResponse(object):
    def __init__(self, parsed_json):
        self._chore_id = parse_int(parsed_json.get("chore_id"), None)
        self._last_tracked_time = parse_date(parsed_json.get("last_tracked_time"))
        self._next_estimated_execution_time = parse_date(
            parsed_json.get("next_estimated_execution_time")
        )

    @property
    def chore_id(self) -> int:
        return self._chore_id

    @property
    def last_tracked_time(self) -> datetime:
        return self._last_tracked_time

    @property
    def next_estimated_execution_time(self) -> datetime:
        return self._next_estimated_execution_time


class CurrentStockResponse(object):
    def __init__(self, parsed_json):
        self._product_id = parse_int(parsed_json.get("product_id"))
        self._amount = parse_float(parsed_json.get("amount"))
        self._best_before_date = parse_date(parsed_json.get("best_before_date"))
        self._amount_opened = parse_float(parsed_json.get("amount_opened"))
        self._product = ProductData(parsed_json.get("product"))

    @property
    def product_id(self) -> int:
        return self._product_id

    @property
    def amount(self) -> float:
        return self._amount

    @property
    def best_before_date(self) -> datetime:
        return self._best_before_date

    @property
    def amount_opened(self) -> float:
        return self._amount_opened

    @property
    def product(self) -> ProductData:
        return self._product


class MissingProductResponse(object):
    def __init__(self, parsed_json):
        self._product_id = parse_int(parsed_json.get("id"))
        self._name = parsed_json.get("name")
        self._amount_missing = parse_float(parsed_json.get("amount_missing"))
        self._is_partly_in_stock = bool(
            parse_int(parsed_json.get("is_partly_in_stock"))
        )

    @property
    def product_id(self) -> int:
        return self._product_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def amount_missing(self) -> float:
        return self._amount_missing

    @property
    def is_partly_in_stock(self) -> bool:
        return self._is_partly_in_stock


class CurrentVolatilStockResponse(object):
    def __init__(self, parsed_json):
        self._due_products = []
        if "due_products" in parsed_json:
            self._due_products = [
                CurrentStockResponse(product)
                for product in parsed_json.get("due_products")
            ]

        self._overdue_products = []
        if "overdue_products" in parsed_json:
            self._overdue_products = [
                CurrentStockResponse(product)
                for product in parsed_json.get("overdue_products")
            ]

        self._expired_products = []
        if "expired_products" in parsed_json:
            self._expired_products = [
                CurrentStockResponse(product)
                for product in parsed_json.get("expired_products")
            ]

        self._missing_products = []
        if "missing_products" in parsed_json:
            self._missing_products = [
                MissingProductResponse(product)
                for product in parsed_json.get("missing_products")
            ]

    @property
    def due_products(self) -> List[CurrentStockResponse]:
        return self._due_products

    @property
    def overdue_products(self) -> List[CurrentStockResponse]:
        return self._overdue_products

    @property
    def expired_products(self) -> List[CurrentStockResponse]:
        return self._expired_products

    @property
    def missing_products(self) -> List[MissingProductResponse]:
        return self._missing_products


class ProductBarcode(object):
    def __init__(self, parsed_json):
        self._barcode = parsed_json.get("barcode")

    @property
    def barcode(self) -> str:
        return self._barcode


class ProductDetailsResponse(object):
    def __init__(self, parsed_json):
        self._last_purchased = parse_date(parsed_json.get("last_purchased"))
        self._last_used = parse_date(parsed_json.get("last_used"))
        self._stock_amount = parse_int(parsed_json.get("stock_amount"))
        self._stock_amount_opened = parse_int(parsed_json.get("stock_amount_opened"))
        self._next_best_before_date = parse_date(
            parsed_json.get("next_best_before_date")
        )
        self._last_price = parse_float(parsed_json.get("last_price"))

        self._product = ProductData(parsed_json.get("product"))

        self._quantity_unit_stock = QuantityUnitData(
            parsed_json.get("quantity_unit_stock")
        )

        self._parse_location(parsed_json)

        self._parse_barcodes(parsed_json)

    def _parse_barcodes(self, parsed_json):
        barcodes_raw = parsed_json.get("product_barcodes", "")
        if barcodes_raw is not None:
            self._barcodes = [ProductBarcode(barcode) for barcode in barcodes_raw]

    def _parse_location(self, parsed_json):
        raw_location = parsed_json.get("location")
        if raw_location is None:
            self._location = None
        else:
            self._location = LocationData(raw_location)

    @property
    def last_purchased(self) -> datetime:
        return self._last_purchased

    @property
    def last_used(self) -> datetime:
        return self._last_used

    @property
    def stock_amount(self) -> int:
        return self._stock_amount

    @property
    def stock_amount_opened(self) -> int:
        return self._stock_amount_opened

    @property
    def next_best_before_date(self) -> datetime:
        return self._next_best_before_date

    @property
    def last_price(self) -> float:
        return self._last_price

    @property
    def barcodes(self) -> List[ProductBarcode]:
        return self._barcodes

    @property
    def product(self) -> ProductData:
        return self._product


class ChoreDetailsResponse(object):
    def __init__(self, parsed_json):
        self._chore = ChoreData(parsed_json.get("chore"))
        self._last_tracked = parse_date(parsed_json.get("last_tracked"))
        self._next_estimated_execution_time = parse_date(
            parsed_json.get("next_estimated_execution_time")
        )
        self._track_count = parse_int(parsed_json.get("track_count"))

        next_user = parsed_json.get("next_execution_assigned_user")
        if next_user is not None:
            self._next_execution_assigned_user = UserDto(next_user)
        else:
            self._next_execution_assigned_user = None

        if self._last_tracked is None:
            self._last_done_by = None
        else:
            self._last_done_by = UserDto(parsed_json.get("last_done_by"))

    @property
    def chore(self) -> ChoreData:
        return self._chore

    @property
    def last_done_by(self) -> UserDto:
        return self._last_done_by

    @property
    def last_tracked(self) -> datetime:
        return self._last_tracked

    @property
    def next_estimated_execution_time(self) -> datetime:
        return self._next_estimated_execution_time

    @property
    def track_count(self) -> int:
        return self._track_count

    @property
    def next_execution_assigned_user(self) -> UserDto:
        return self._next_execution_assigned_user


class TransactionType(Enum):
    PURCHASE = "purchase"
    CONSUME = "consume"
    INVENTORY_CORRECTION = "inventory-correction"
    PRODUCT_OPENED = "product-opened"


class TaskResponse(object):
    def __init__(self, parsed_json):
        self.id = parse_int(parsed_json.get("id"))
        self.name = parsed_json.get("name")
        self.description = parsed_json.get("description")
        self.due_date = parse_date(parsed_json.get("due_date"))
        self.done = parse_int(parsed_json.get("done"))
        self.done_timestamp = parse_date(parsed_json.get("done_timestamp"))
        self.category_id = parse_int(parsed_json.get("category_id"))
        self.assigned_to_user_id = parse_int(parsed_json.get("assigned_to_user_id"))
        self.userfields = parsed_json.get("userfields")


class CurrentBatteryResponse(object):
    def __init__(self, parsed_json):
        self.id = parse_int(parsed_json.get("battery_id"))
        self.last_tracked_time = parse_date(parsed_json.get("last_tracked_time"))
        self.next_estimated_charge_time = parse_date(
            parsed_json.get("'next_estimated_charge_time")
        )


class BatteryData(object):
    def __init__(self, parsed_json):
        self.id = parse_int(parsed_json.get("id"))
        self.name = parsed_json.get("name")
        self.description = parsed_json.get("description")
        self.used_in = parsed_json.get("used_in")
        self.charge_interval_days = parse_int(parsed_json.get("charge_interval_days"))
        self.created_timestamp = parse_date(parsed_json.get("row_created_timestamp"))
        self.userfields = parsed_json.get("userfields")


class BatteryDetailsResponse(object):
    def __init__(self, parsed_json):
        self.battery = BatteryData(parsed_json.get("battery"))
        self.charge_cycles_count = parse_int(parsed_json.get("charge_cycles_count"))
        self.last_charged = parse_date(parsed_json.get("last_charged"))
        self.next_estimated_charge_time = parse_date(
            parsed_json.get("'next_estimated_charge_time")
        )


class GrocyApiClient(object):
    def __init__(
        self, base_url, api_key, port: int = DEFAULT_PORT_NUMBER, verify_ssl=True
    ):
        self._base_url = "{}:{}/api/".format(base_url, port)
        self._api_key = api_key
        self._verify_ssl = verify_ssl
        if self._api_key == "demo_mode":
            self._headers = {"accept": "application/json"}
        else:
            self._headers = {"accept": "application/json", "GROCY-API-KEY": api_key}

    def _do_get_request(self, end_url: str):
        req_url = urljoin(self._base_url, end_url)
        resp = requests.get(req_url, verify=self._verify_ssl, headers=self._headers)
        resp.raise_for_status()
        if len(resp.content) > 0:
            return resp.json()

    def _do_post_request(self, end_url: str, data: dict):
        req_url = urljoin(self._base_url, end_url)
        resp = requests.post(
            req_url, verify=self._verify_ssl, headers=self._headers, json=data
        )
        resp.raise_for_status()
        if len(resp.content) > 0:
            return resp.json()

    def _do_put_request(self, end_url: str, data):
        req_url = urljoin(self._base_url, end_url)
        up_header = self._headers.copy()
        up_header["accept"] = "*/*"
        if isinstance(data, dict):
            up_header["Content-Type"] = "application/json"
            data = json.dumps(data)
        else:
            up_header["Content-Type"] = "application/octet-stream"
        resp = requests.put(
            req_url, verify=self._verify_ssl, headers=up_header, data=data
        )
        resp.raise_for_status()
        if len(resp.content) > 0:
            return resp.json()

    def get_stock(self) -> List[CurrentStockResponse]:
        parsed_json = self._do_get_request("stock")
        return [CurrentStockResponse(response) for response in parsed_json]

    def get_volatile_stock(self) -> CurrentVolatilStockResponse:
        parsed_json = self._do_get_request("stock/volatile")
        return CurrentVolatilStockResponse(parsed_json)

    def get_product(self, product_id) -> ProductDetailsResponse:
        url = f"stock/products/{product_id}"
        parsed_json = self._do_get_request(url)
        if parsed_json:
            return ProductDetailsResponse(parsed_json)

    def get_chores(self) -> List[CurrentChoreResponse]:
        parsed_json = self._do_get_request("chores")
        return [CurrentChoreResponse(chore) for chore in parsed_json]

    def get_chore(self, chore_id: int) -> ChoreDetailsResponse:
        url = f"chores/{chore_id}"
        parsed_json = self._do_get_request(url)
        if parsed_json:
            return ChoreDetailsResponse(parsed_json)

    def execute_chore(
        self,
        chore_id: int,
        done_by: int = None,
        tracked_time: datetime = datetime.now(),
    ):
        localized_tracked_time = localize_datetime(tracked_time)

        data = {"tracked_time": localized_tracked_time.isoformat()}

        if done_by is not None:
            data["done_by"] = done_by

        return self._do_post_request(f"chores/{chore_id}/execute", data)

    def add_product(
        self,
        product_id,
        amount: float,
        price: float,
        best_before_date: datetime = None,
        transaction_type: TransactionType = TransactionType.PURCHASE,
    ):
        data = {
            "amount": amount,
            "transaction_type": transaction_type.value,
            "price": price,
        }

        if best_before_date is not None:
            data["best_before_date"] = best_before_date.strftime("%Y-%m-%d")

        return self._do_post_request(f"stock/products/{product_id}/add", data)

    def consume_product(
        self,
        product_id: int,
        amount: float = 1,
        spoiled: bool = False,
        transaction_type: TransactionType = TransactionType.CONSUME,
    ):
        data = {
            "amount": amount,
            "spoiled": spoiled,
            "transaction_type": transaction_type.value,
        }

        self._do_post_request(f"stock/products/{product_id}/consume", data)

    def get_shopping_list(self) -> List[ShoppingListItem]:
        parsed_json = self._do_get_request("objects/shopping_list")
        return [ShoppingListItem(response) for response in parsed_json]

    def add_missing_product_to_shopping_list(self, shopping_list_id: int = None):
        data = None
        if shopping_list_id:
            data = {"list_id": shopping_list_id}

        self._do_post_request("stock/shoppinglist/add-missing-products", data)

    def add_product_to_shopping_list(
        self, product_id: int, shopping_list_id: int = 1, amount: int = 1
    ):
        data = {
            "product_id": product_id,
            "list_id": shopping_list_id,
            "product_amount": amount,
        }
        self._do_post_request("stock/shoppinglist/add-product", data)

    def clear_shopping_list(self, shopping_list_id: int = 1):
        data = {"list_id": shopping_list_id}

        self._do_post_request("stock/shoppinglist/clear", data)

    def remove_product_in_shopping_list(
        self, product_id: int, shopping_list_id: int = 1, amount: int = 1
    ):
        data = {
            "product_id": product_id,
            "list_id": shopping_list_id,
            "product_amount": amount,
        }
        self._do_post_request("stock/shoppinglist/remove-product", data)

    def get_product_groups(self) -> List[LocationData]:
        parsed_json = self._do_get_request("objects/product_groups")
        return [LocationData(response) for response in parsed_json]

    def upload_product_picture(self, product_id: int, pic_path: str):
        b64fn = base64.b64encode("{}.jpg".format(product_id).encode("ascii"))
        req_url = "files/productpictures/" + str(b64fn, "utf-8")
        with open(pic_path, "rb") as pic:
            self._do_put_request(req_url, pic)

    def update_product_pic(self, product_id: int):
        pic_name = f"{product_id}.jpg"
        data = {"picture_file_name": pic_name}
        self._do_put_request(f"objects/products/{product_id}", data)

    def get_userfields(self, entity: str, object_id: int):
        url = f"userfields/{entity}/{object_id}"
        return self._do_get_request(url)

    def set_userfields(self, entity: str, object_id: int, key: str, value):
        data = {key: value}
        self._do_put_request(f"userfields/{entity}/{object_id}", data)

    def get_last_db_changed(self):
        resp = self._do_get_request("system/db-changed-time")
        last_change_timestamp = parse_date(resp.get("changed_time"))
        return last_change_timestamp

    def get_tasks(self) -> List[TaskResponse]:
        parsed_json = self._do_get_request("tasks")
        return [TaskResponse(data) for data in parsed_json]

    def complete_task(self, task_id: int, done_time: datetime = datetime.now()):
        url = f"tasks/{task_id}/complete"

        localized_done_time = localize_datetime(done_time)

        data = {"done_time": localized_done_time.isoformat()}
        self._do_post_request(url, data)

    def get_meal_plan(self) -> List[MealPlanResponse]:
        parsed_json = self._do_get_request("objects/meal_plan")
        return [MealPlanResponse(data) for data in parsed_json]

    def get_recipe(self, object_id: int) -> RecipeDetailsResponse:
        parsed_json = self._do_get_request(f"objects/recipes/{object_id}")
        if parsed_json:
            return RecipeDetailsResponse(parsed_json)

    def add_generic(self, entity_type: str, data: object):
        self._do_post_request(f"objects/{entity_type}", data)

    def get_batteries(self) -> List[CurrentBatteryResponse]:
        parsed_json = self._do_get_request(f"batteries")
        if parsed_json:
            return [CurrentBatteryResponse(data) for data in parsed_json]

    def get_battery(self, battery_id: int) -> BatteryDetailsResponse:
        parsed_json = self._do_get_request(f"batteries/{battery_id}")
        if parsed_json:
            return BatteryDetailsResponse(parsed_json)

    def charge_battery(self, battery_id: int, tracked_time: datetime = datetime.now()):
        localized_tracked_time = localize_datetime(tracked_time)
        data = {"tracked_time": localized_tracked_time.isoformat()}

        return self._do_post_request(f"batteries/{battery_id}/charge", data)



def get_val(obj):
    if hasattr(obj, 'as_dict'):
        as_attr = getattr(obj, 'as_dict')
        return as_attr()
    return obj


class DataModel(object):
    def as_dict(self):
        return {k: get_val(getattr(self, k)) for k, v in self.__class__.__dict__.items() if isinstance(v, property)}



import pytest


class TestTasks:
    @pytest.mark.vcr
    def test_get_tasks_valid(self, grocy):
        tasks = grocy.tasks()

        assert len(tasks) == 6
        assert tasks[0].id == 1
        assert tasks[0].name == "Repair the garage door"



from datetime import datetime

import pytest


class TestBattery:
    @pytest.mark.vcr
    def test_get_batteries_valid(self, grocy):
        batteries = grocy.batteries()

        assert len(batteries) == 4
        assert isinstance(batteries[0].last_tracked_time, datetime)

    @pytest.mark.vcr
    def test_get_battery_details_valid(self, grocy):
        battery = grocy.battery(1)

        assert battery.id == 1
        assert battery.name == "Battery1"
        assert battery.description == "Warranty ends 2023"
        assert battery.used_in == "TV remote control"
        assert battery.charge_interval_days == 0
        assert isinstance(battery.created_timestamp, datetime)
        assert isinstance(battery.last_charged, datetime)
        assert battery.next_estimated_charge_time is None
        assert battery.userfields is None
        assert battery.charge_cycles_count == 4

    @pytest.mark.vcr
    def test_charge_battery(self, grocy):
        assert grocy.charge_battery(1)



import json
from unittest import TestCase

from pygrocy.grocy_api_client import MissingProductResponse


class TestMissingProductResponse(TestCase):
    def test_parse_partly_in_stock_true(self):
        input_json = """{ "id": "7", "name": "XXXX", "amount_missing": "2", "is_partly_in_stock": "1" }"""
        response = MissingProductResponse(json.loads(input_json))

        assert response.product_id == 7
        assert response.name == "XXXX"
        assert response.amount_missing == 2
        assert response.is_partly_in_stock

    def test_parse_partly_in_stock_false(self):
        input_json = """{ "id": "7", "name": "XXXX", "amount_missing": "2", "is_partly_in_stock": "0" }"""
        response = MissingProductResponse(json.loads(input_json))

        assert response.product_id == 7
        assert response.name == "XXXX"
        assert response.amount_missing == 2
        assert not response.is_partly_in_stock



from datetime import datetime
from unittest import TestCase

import pygrocy.utils as utils


class TestUtils(TestCase):
    def test_parse_date_valid(self):
        date_str = "2019-05-04T11:31:04.563Z"
        date_obj = utils.parse_date(date_str)

        assert isinstance(date_obj, datetime)

    def test_parse_date_no_data(self):
        date_str = None
        date_obj = utils.parse_date(date_str)

        assert date_obj is None
        
    def test_parse_date_empty_string(self):
        date_str = ""
        date_obj = utils.parse_date(date_str)

        assert date_obj is None
       
    def test_parse_int_valid(self):
        int_str = "2"
        int_number = utils.parse_int(int_str)

        assert isinstance(int_number, int)

    def test_parse_int_no_data(self):
        int_str = None
        int_number = utils.parse_int(int_str, -1)

        assert int_number == -1

    def test_parse_int_error(self):
        int_str = "string"
        int_number = utils.parse_int(int_str, -1)

        assert int_number == -1

    def test_parse_float_valid(self):
        float_str = "2.01"
        float_number = utils.parse_float(float_str)

        assert isinstance(float_number, float)

    def test_parse_float_no_data(self):
        float_str = None
        float_number = utils.parse_float(float_str, -1)

        assert float_number == -1

    def test_parse_float_error(self):
        float_str = "string"
        float_number = utils.parse_float(float_str, -1)

        assert float_number == -1



import json
from datetime import datetime
from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
from unittest import TestCase
from unittest.mock import mock_open, patch

import responses
from requests.exceptions import HTTPError

from pygrocy import Grocy
from pygrocy.grocy_api_client import GrocyApiClient


class TestGrocy(TestCase):
    def setUp(self):
        self.grocy_regular = Grocy(CONST_BASE_URL, "api_key")
        self.grocy = Grocy(
            CONST_BASE_URL, "demo_mode", verify_ssl=CONST_SSL, port=CONST_PORT
        )
        self.base_url = f"{CONST_BASE_URL}:{CONST_PORT}/api"
        self.date_test = datetime.strptime("2019-05-04 11:31:04", "%Y-%m-%d %H:%M:%S")
        self.add_generic_data = {"name": "This is a task"}

    @responses.activate
    def test_get_shopping_list_invalid_no_data(self):
        responses.add(
            responses.GET, f"{self.base_url}/objects/shopping_list", status=400
        )
        self.assertRaises(HTTPError, self.grocy.shopping_list)

    @responses.activate
    def test_get_shopping_list_invalid_missing_data(self):
        resp = []
        responses.add(
            responses.GET,
            f"{self.base_url}/objects/shopping_list",
            json=resp,
            status=200,
        )
        self.assertEqual(len(self.grocy.shopping_list()), 0)

    @responses.activate
    def test_add_missing_product_to_shopping_list_error(self):
        responses.add(
            responses.POST,
            f"{self.base_url}/stock/shoppinglist/add-missing-products",
            status=400,
        )
        self.assertRaises(HTTPError, self.grocy.add_missing_product_to_shopping_list)

    @responses.activate
    def test_clear_shopping_list_error(self):
        responses.add(
            responses.POST, f"{self.base_url}/stock/shoppinglist/clear", status=400
        )
        self.assertRaises(HTTPError, self.grocy.clear_shopping_list)

    @responses.activate
    def test_remove_product_in_shopping_list_error(self):
        responses.add(
            responses.POST,
            f"{self.base_url}/stock/shoppinglist/remove-product",
            status=400,
        )
        self.assertRaises(HTTPError, self.grocy.remove_product_in_shopping_list, 1)

    @responses.activate
    def test_get_product_groups_invalid_no_data(self):
        responses.add(
            responses.GET, f"{self.base_url}/objects/product_groups", status=400
        )
        self.assertRaises(HTTPError, self.grocy.product_groups)

    @responses.activate
    def test_get_product_groups_invalid_missing_data(self):
        resp = []
        responses.add(
            responses.GET,
            f"{self.base_url}/objects/product_groups",
            json=resp,
            status=200,
        )
        self.assertEqual(len(self.grocy.product_groups()), 0)

    @responses.activate
    def test_add_product_pic_invalid_missing_data(self):
        with patch("os.path.exists") as m_exist:
            m_exist.return_value = False
            self.assertRaises(
                FileNotFoundError, self.grocy.add_product_pic, 1, "/somepath/pic.jpg"
            )

    @responses.activate
    def test_upload_product_picture_error(self):
        with patch("os.path.exists") as m_exist:
            with patch("builtins.open", mock_open()):
                m_exist.return_value = True
                api_client = GrocyApiClient(
                    CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL
                )
                responses.add(
                    responses.PUT,
                    f"{self.base_url}/files/productpictures/MS5qcGc=",
                    status=400,
                )
                self.assertRaises(
                    HTTPError, api_client.upload_product_picture, 1, "/somepath/pic.jpg"
                )

    @responses.activate
    def test_update_product_pic_error(self):
        api_client = GrocyApiClient(
            CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL
        )
        responses.add(responses.PUT, f"{self.base_url}/objects/products/1", status=400)
        self.assertRaises(HTTPError, api_client.update_product_pic, 1)

    @responses.activate
    def test_get_due_invalid_no_data(self):
        resp = {"due_products": [], "expired_products": [], "missing_products": []}
        responses.add(
            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
        )

        self.grocy.due_products(True)

    @responses.activate
    def test_get_expired_invalid_no_data(self):
        resp = {"expiring_products": [], "expired_products": [], "missing_products": []}
        responses.add(
            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
        )

        self.grocy.expired_products(True)

    @responses.activate
    def test_get_expired_invalid_missing_data(self):
        resp = {}
        responses.add(
            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
        )

    @responses.activate
    def test_get_missing_invalid_no_data(self):
        resp = {"expiring_products": [], "expired_products": [], "missing_products": []}
        responses.add(
            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
        )

        self.grocy.missing_products(True)

    @responses.activate
    def test_get_missing_invalid_missing_data(self):
        resp = {}
        responses.add(
            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
        )

    @responses.activate
    def test_get_userfields_valid(self):
        resp = {"uf1": 0, "uf2": "string"}

        responses.add(
            responses.GET, f"{self.base_url}/userfields/chores/1", json=resp, status=200
        )

        a_chore_uf = self.grocy.get_userfields("chores", 1)

        self.assertEqual(a_chore_uf["uf1"], 0)

    @responses.activate
    def test_set_userfields_valid(self):
        responses.add(responses.PUT, f"{self.base_url}/userfields/chores/1", status=204)
        self.grocy.set_userfields("chores", 1, "auserfield", "value")

    @responses.activate
    def test_set_userfields_error(self):
        responses.add(responses.PUT, f"{self.base_url}/userfields/chores/1", status=400)
        self.assertRaises(
            HTTPError, self.grocy.set_userfields, "chores", 1, "auserfield", "value"
        )

    @responses.activate
    def test_get_last_db_changed_invalid_no_data(self):
        resp = {}
        responses.add(
            responses.GET,
            f"{self.base_url}/system/db-changed-time",
            json=resp,
            status=200,
        )

        self.assertIsNone(self.grocy.get_last_db_changed())

    @responses.activate
    def test_add_product_valid(self):
        responses.add(
            responses.POST, f"{self.base_url}/stock/products/1/add", status=200
        )
        self.assertIsNone(self.grocy.add_product(1, 1.3, 2.44, self.date_test))

    @responses.activate
    def test_add_product_error(self):
        responses.add(
            responses.POST, f"{self.base_url}/stock/products/1/add", status=400
        )
        self.assertRaises(
            HTTPError, self.grocy.add_product, 1, 1.3, 2.44, self.date_test
        )

    @responses.activate
    def test_add_generic_valid(self):
        responses.add(responses.POST, f"{self.base_url}/objects/tasks", status=200)
        self.assertIsNone(self.grocy.add_generic("tasks", self.add_generic_data))

    @responses.activate
    def test_add_generic_error(self):
        responses.add(responses.POST, f"{self.base_url}/objects/tasks", status=400)
        self.assertRaises(
            HTTPError, self.grocy.add_generic, "tasks", self.add_generic_data
        )

    @responses.activate
    def test_consume_product_valid(self):
        responses.add(
            responses.POST, f"{self.base_url}/stock/products/1/consume", status=200
        )
        self.assertIsNone(self.grocy.consume_product(1, 1.3))

    @responses.activate
    def test_consume_product_error(self):
        responses.add(
            responses.POST, f"{self.base_url}/stock/products/1/consume", status=400
        )
        self.assertRaises(HTTPError, self.grocy.consume_product, 1, 1.3)

    @responses.activate
    def test_execute_chore_valid(self):
        responses.add(responses.POST, f"{self.base_url}/chores/1/execute", status=200)
        self.assertIsNone(self.grocy.execute_chore(1, 1, self.date_test))

    @responses.activate
    def test_execute_chore_error(self):
        responses.add(responses.POST, f"{self.base_url}/chores/1/execute", status=400)
        self.assertRaises(HTTPError, self.grocy.execute_chore, 1, 1, self.date_test)

    @responses.activate
    def test_get_meal_plan(self):
        resp_json = json.loads(
            """[
              {
                "id": "1",
                "day": "2020-08-10",
                "type": "recipe",
                "recipe_id": "1",
                "recipe_servings": "1",
                "note": null,
                "product_id": null,
                "product_amount": "0.0",
                "product_qu_id": null,
                "row_created_timestamp": "2020-08-12 19:59:30",
                "userfields": null
              }
          ]"""
        )
        responses.add(
            responses.GET,
            f"{self.base_url}/objects/meal_plan",
            json=resp_json,
            status=200,
        )
        meal_plan = self.grocy.meal_plan()
        self.assertEqual(len(meal_plan), 1)
        self.assertEqual(meal_plan[0].id, 1)
        self.assertEqual(meal_plan[0].recipe_id, 1)

    @responses.activate
    def test_get_recipe(self):
        resp_json = json.loads(
            """{
          "id": "1",
          "name": "Pizza",
          "description": "<p>Mix everything</p>",
          "row_created_timestamp": "2020-08-12 11:37:34",
          "picture_file_name": "51si0q0wsiq5imo4f8wbIMG_5709.jpeg",
          "base_servings": "4",
          "desired_servings": "4",
          "not_check_shoppinglist": "0",
          "type": "normal",
          "product_id": "",
          "userfields": null
        }"""
        )
        responses.add(
            responses.GET,
            f"{self.base_url}/objects/recipes/1",
            json=resp_json,
            status=200,
        )
        recipe = self.grocy.recipe(1)
        self.assertEqual(recipe.id, 1)
        self.assertEqual(recipe.name, "Pizza")
        self.assertEqual(recipe.base_servings, 4)
        self.assertIsInstance(recipe.get_picture_url_path(400), str)



import json
from unittest import TestCase

from pygrocy.grocy_api_client import CurrentChoreResponse


class TestCurrentChoreResponse(TestCase):
    def test_parse(self):
        input_json = """{ "chore_id": "4", "last_tracked_time": null, "next_estimated_execution_time": "2999-12-31 23:59:59", "track_date_only": "0", "next_execution_assigned_to_user_id": null }"""

        response = CurrentChoreResponse(json.loads(input_json))

        assert response.chore_id == 4
    



from datetime import datetime

import iso8601
import pytz
from tzlocal import get_localzone


def parse_date(input_value):
    if input_value == "" or input_value is None:
        return None
    return iso8601.parse_date(input_value)


def parse_int(input_value, default_value=None):
    if input_value is None:
        return default_value
    try:
        return int(input_value)
    except ValueError:
        return default_value


def parse_float(input_value, default_value=None):
    if input_value is None:
        return default_value
    try:
        return float(input_value)
    except ValueError:
        return default_value


def parse_bool_int(input_value):
    if input_value is None:
        return False
    try:
        num = int(input_value)
        return bool(num)
    except ValueError:
        return False


def localize_datetime(timestamp: datetime) -> datetime:
    if timestamp.tzinfo is not None:
        return timestamp

    local_tz = get_localzone()
    return local_tz.localize(timestamp).astimezone(pytz.utc)



import base64
from datetime import datetime

from pygrocy.base import DataModel
from pygrocy.grocy_api_client import (
    GrocyApiClient,
    MealPlanResponse,
    RecipeDetailsResponse,
)


class RecipeItem(DataModel):
    def __init__(self, response: RecipeDetailsResponse):
        self._id = response.id
        self._name = response.name
        self._description = response.description
        self._base_servings = response.base_servings
        self._desired_servings = response.desired_servings
        self._picture_file_name = response.picture_file_name

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def base_servings(self) -> int:
        return self._base_servings

    @property
    def desired_servings(self) -> int:
        return self._desired_servings

    @property
    def picture_file_name(self) -> str:
        return self._picture_file_name

    def get_picture_url_path(self, width: int = 400):
        if self.picture_file_name:
            b64name = base64.b64encode(self.picture_file_name.encode("ascii"))
            path = "files/recipepictures/" + str(b64name, "utf-8")

            return f"{path}?force_serve_as=picture&best_fit_width={width}"


class MealPlanItem(DataModel):
    def __init__(self, response: MealPlanResponse):
        self._id = response.id
        self._day = response.day
        self._recipe = None
        self._recipe_id = response.recipe_id
        self._recipe_servings = response.recipe_servings
        self._note = response.note

    @property
    def id(self) -> int:
        return self._id

    @property
    def day(self) -> datetime:
        return self._day

    @property
    def recipe_id(self) -> int:
        return self._recipe_id

    @property
    def recipe_servings(self) -> int:
        return self._recipe_servings

    @property
    def note(self) -> str:
        return self._note

    @property
    def recipe(self) -> RecipeItem:
        return self._recipe

    def get_details(self, api_client: GrocyApiClient):
        if self.recipe_id:
            recipe = api_client.get_recipe(self.recipe_id)
            if recipe:
                self._recipe = RecipeItem(recipe)



import pytest

from pygrocy.data_models.product import Product


class TestStock:
    @pytest.mark.vcr
    def test_get_stock_valid(self, grocy):
        stock = grocy.stock()

        assert isinstance(stock, list)
        assert len(stock) == 19
        for prod in stock:
            assert isinstance(prod, Product)

    @pytest.mark.vcr
    def test_get_due_products_valid(self, grocy):
        due_products = grocy.due_products(True)

        assert isinstance(due_products, list)
        assert len(due_products) == 3
        for prod in due_products:
            assert isinstance(prod, Product)

    @pytest.mark.vcr
    def test_get_expired_products_valid(self, grocy):
        expired_products = grocy.expired_products(True)

        assert isinstance(expired_products, list)
        assert len(expired_products) == 1
        for prod in expired_products:
            assert isinstance(prod, Product)

    @pytest.mark.vcr
    def test_get_missing_products_valid(self, grocy):
        missing_products = grocy.missing_products(True)

        assert isinstance(missing_products, list)
        assert len(missing_products) == 4
        for prod in missing_products:
            assert isinstance(prod, Product)
            assert isinstance(prod.amount_missing, float)
            assert isinstance(prod.is_partly_in_stock, bool)

        product = next(product for product in missing_products if product.id == 1)
        assert product.is_partly_in_stock is False
        assert product.amount_missing == 8.0

    @pytest.mark.vcr
    def test_get_overdue_products_valid(self, grocy):
        overdue_products = grocy.overdue_products(True)

        assert isinstance(overdue_products, list)
        assert len(overdue_products) == 2
        for prod in overdue_products:
            assert isinstance(prod, Product)



from datetime import datetime

from pygrocy.base import DataModel
from pygrocy.grocy_api_client import BatteryDetailsResponse, CurrentBatteryResponse


class Battery(DataModel):
    def __init__(self, response):
        self._init_empty()

        self._next_estimated_charge_time = response.next_estimated_charge_time

        if isinstance(response, CurrentBatteryResponse):
            self._init_from_CurrentBatteryResponse(response)
        elif isinstance(response, BatteryDetailsResponse):
            self._init_from_BatteryDetailsResponse(response)

    def _init_from_CurrentBatteryResponse(self, response: CurrentBatteryResponse):
        self._id = response.id
        self._last_tracked_time = response.last_tracked_time

    def _init_from_BatteryDetailsResponse(self, response: BatteryDetailsResponse):
        self._charge_cycles_count = response.charge_cycles_count
        self._last_charged = response.last_charged
        self._id = response.battery.id
        self._name = response.battery.name
        self._description = response.battery.description
        self._used_in = response.battery.used_in
        self._charge_interval_days = response.battery.charge_interval_days
        self._created_timestamp = response.battery.created_timestamp
        self._userfields = response.battery.userfields

    def _init_empty(self):
        self._last_tracked_time = None
        self._charge_cycles_count = None
        self._last_charged = None
        self._name = None
        self._description = None
        self._used_in = None
        self._charge_interval_days = None
        self._created_timestamp = None
        self._userfields = None

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def used_in(self) -> str:
        return self._used_in

    @property
    def charge_interval_days(self) -> int:
        return self._charge_interval_days

    @property
    def created_timestamp(self) -> datetime:
        return self._created_timestamp

    @property
    def charge_cycles_count(self) -> int:
        return self._charge_cycles_count

    @property
    def userfields(self):
        return self._userfields

    @property
    def last_charged(self) -> datetime:
        return self._last_charged

    @property
    def last_tracked_time(self) -> datetime:
        return self._last_tracked_time

    @property
    def next_estimated_charge_time(self) -> datetime:
        return self._next_estimated_charge_time



import json
from unittest import TestCase

from pygrocy.grocy_api_client import ProductDetailsResponse


class TestProductDetailsResponse(TestCase):
    def test_barcode_null(self):
        input_json = """{ "product": { "id": 0, "name": "string", "description": "string", "location_id": 0, "qu_id_purchase": 0, "qu_id_stock": 0, "qu_factor_purchase_to_stock": 0, "barcode": null, "min_stock_amount": 0, "default_best_before_days": 0, "picture_file_name": "string", "allow_partial_units_in_stock": true, "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "quantity_unit_purchase": { "id": 0, "name": "string", "name_plural": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "quantity_unit_stock": { "id": 0, "name": "string", "name_plural": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "last_purchased": "2019-04-22", "last_used": "2019-04-22T09:54:15.835Z", "stock_amount": 10, "stock_amount_opened": 2, "next_best_before_date": "2019-04-22T09:54:15.835Z", "last_price": 0, "location": { "id": 0, "name": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" } }"""
        response = ProductDetailsResponse(json.loads(input_json))

        assert len(response.barcodes) == 0

    def test_parse(self):
        input_json = """{ "product_barcodes": [{"barcode":"string"},{"barcode":"123"}], "product": { "id": 0, "name": "string", "description": "string", "location_id": 0, "product_group_id": 0, "qu_id_purchase": 0, "qu_id_stock": 0, "qu_factor_purchase_to_stock": 0, "barcode": "string,123", "min_stock_amount": 0, "default_best_before_days": 0, "picture_file_name": "string", "allow_partial_units_in_stock": true, "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "quantity_unit_purchase": { "id": 0, "name": "string", "name_plural": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "quantity_unit_stock": { "id": 0, "name": "string", "name_plural": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" }, "last_purchased": "2019-04-22", "last_used": "2019-04-22T09:54:15.835Z", "stock_amount": 10, "stock_amount_opened": 2, "next_best_before_date": "2019-04-22T09:54:15.835Z", "last_price": 0, "location": { "id": 0, "name": "string", "description": "string", "row_created_timestamp": "2019-04-22T09:54:15.835Z" } }"""
        response = ProductDetailsResponse(json.loads(input_json))

        assert response.stock_amount == 10
        assert response.stock_amount_opened == 2
        assert response.last_price == 0
        assert response.product.product_group_id == 0

        assert response.next_best_before_date.year == 2019
        assert response.next_best_before_date.month == 4
        assert response.next_best_before_date.day == 22
        assert response.next_best_before_date.hour == 9
        assert response.next_best_before_date.minute == 54
        assert response.next_best_before_date.second == 15

        assert response.last_used.year == 2019
        assert response.last_used.month == 4
        assert response.last_used.day == 22
        assert response.last_used.hour == 9
        assert response.last_used.minute == 54
        assert response.last_used.second == 15

        assert response.last_purchased.year == 2019
        assert response.last_purchased.month == 4
        assert response.last_purchased.day == 22

        assert [barcode.barcode for barcode in response.barcodes] == ["string", "123"]



from datetime import datetime

import pytest

from pygrocy.data_models.chore import AssignmentType, Chore, PeriodType
from pygrocy.data_models.user import User


class TestChores:
    @pytest.mark.vcr
    def test_get_chores_valid(self, grocy):
        chores = grocy.chores(get_details=True)

        assert isinstance(chores, list)
        assert len(chores) == 6
        for chore in chores:
            assert isinstance(chore, Chore)
            assert isinstance(chore.id, int)
            assert isinstance(chore.last_tracked_time, datetime)
            assert isinstance(chore.next_estimated_execution_time, datetime)
            assert isinstance(chore.name, str)
            assert isinstance(chore.last_done_by, User)

        chore = next(chore for chore in chores if chore.id == 6)
        assert chore.name == "The thing which happens on Mondays and Wednesdays"
        assert chore.period_config == "monday,wednesday"

    @pytest.mark.vcr
    def test_get_chore_details(self, grocy):
        chore_details = grocy.chore(3)
        assert isinstance(chore_details, Chore)

        assert chore_details.name == "Lawn mowed in the garden"
        assert chore_details.assignment_type == AssignmentType.RANDOM
        assert chore_details.last_done_by.id == 1
        assert chore_details.period_type == PeriodType.DYNAMIC_REGULAR
        assert chore_details.period_days == 21
        assert chore_details.period_config is None
        assert chore_details.track_date_only is False
        assert chore_details.rollover is False
        assert chore_details.assignment_config == "1,2,3,4"
        assert chore_details.next_execution_assigned_user.id == 2
        assert chore_details.next_execution_assigned_to_user_id == 2
        assert chore_details.userfields is None



from datetime import datetime
from typing import List

import deprecation

from .base import DataModel  # noqa: F401
from .data_models.battery import Battery
from .data_models.chore import Chore
from .data_models.meal_items import MealPlanItem, RecipeItem
from .data_models.product import Group, Product, ShoppingListProduct
from .data_models.task import Task
from .data_models.user import User  # noqa: F401
from .grocy_api_client import ChoreDetailsResponse  # noqa: F401
from .grocy_api_client import CurrentChoreResponse  # noqa: F401
from .grocy_api_client import CurrentStockResponse  # noqa: F401
from .grocy_api_client import LocationData  # noqa: F401
from .grocy_api_client import MealPlanResponse  # noqa: F401
from .grocy_api_client import MissingProductResponse  # noqa: F401
from .grocy_api_client import ProductDetailsResponse  # noqa: F401
from .grocy_api_client import RecipeDetailsResponse  # noqa: F401
from .grocy_api_client import ShoppingListItem  # noqa: F401
from .grocy_api_client import TaskResponse  # noqa: F401
from .grocy_api_client import UserDto  # noqa: F401
from .grocy_api_client import DEFAULT_PORT_NUMBER, GrocyApiClient, TransactionType


class Grocy(object):
    def __init__(
        self, base_url, api_key, port: int = DEFAULT_PORT_NUMBER, verify_ssl=True
    ):
        self._api_client = GrocyApiClient(base_url, api_key, port, verify_ssl)

    def stock(self) -> List[Product]:
        raw_stock = self._api_client.get_stock()
        stock = [Product(resp) for resp in raw_stock]
        return stock

    @deprecation.deprecated(details="Use due_products instead")
    def expiring_products(self, get_details: bool = False) -> List[Product]:
        return self.due_products(get_details)

    def due_products(self, get_details: bool = False) -> List[Product]:
        raw_due_prodcuts = self._api_client.get_volatile_stock().due_products
        due_products = [Product(resp) for resp in raw_due_prodcuts]

        if get_details:
            for item in due_products:
                item.get_details(self._api_client)
        return due_products

    def overdue_products(self, get_details: bool = False) -> List[Product]:
        raw_overdue_products = self._api_client.get_volatile_stock().overdue_products
        overdue_products = [Product(resp) for resp in raw_overdue_products]

        if get_details:
            for item in overdue_products:
                item.get_details(self._api_client)
        return overdue_products

    def expired_products(self, get_details: bool = False) -> List[Product]:
        raw_expired_products = self._api_client.get_volatile_stock().expired_products
        expired_products = [Product(resp) for resp in raw_expired_products]

        if get_details:
            for item in expired_products:
                item.get_details(self._api_client)
        return expired_products

    def missing_products(self, get_details: bool = False) -> List[Product]:
        raw_missing_products = self._api_client.get_volatile_stock().missing_products
        missing_products = [Product(resp) for resp in raw_missing_products]

        if get_details:
            for item in missing_products:
                item.get_details(self._api_client)
        return missing_products

    def product(self, product_id: int) -> Product:
        resp = self._api_client.get_product(product_id)
        if resp:
            return Product(resp)

    def chores(self, get_details: bool = False) -> List[Chore]:
        raw_chores = self._api_client.get_chores()
        chores = [Chore(chore) for chore in raw_chores]

        if get_details:
            for chore in chores:
                chore.get_details(self._api_client)
        return chores

    def execute_chore(
        self,
        chore_id: int,
        done_by: int = None,
        tracked_time: datetime = datetime.now(),
    ):
        return self._api_client.execute_chore(chore_id, done_by, tracked_time)

    def chore(self, chore_id: int) -> Chore:
        resp = self._api_client.get_chore(chore_id)
        return Chore(resp)

    def add_product(
        self,
        product_id,
        amount: float,
        price: float,
        best_before_date: datetime = None,
        transaction_type: TransactionType = TransactionType.PURCHASE,
    ):
        return self._api_client.add_product(
            product_id, amount, price, best_before_date, transaction_type
        )

    def consume_product(
        self,
        product_id: int,
        amount: float = 1,
        spoiled: bool = False,
        transaction_type: TransactionType = TransactionType.CONSUME,
    ):
        return self._api_client.consume_product(
            product_id, amount, spoiled, transaction_type
        )

    def shopping_list(self, get_details: bool = False) -> List[ShoppingListProduct]:
        raw_shoppinglist = self._api_client.get_shopping_list()
        shopping_list = [ShoppingListProduct(resp) for resp in raw_shoppinglist]

        if get_details:
            for item in shopping_list:
                item.get_details(self._api_client)
        return shopping_list

    def add_missing_product_to_shopping_list(self, shopping_list_id: int = 1):
        return self._api_client.add_missing_product_to_shopping_list(shopping_list_id)

    def add_product_to_shopping_list(
        self, product_id: int, shopping_list_id: int = None, amount: int = None
    ):
        return self._api_client.add_product_to_shopping_list(
            product_id, shopping_list_id, amount
        )

    def clear_shopping_list(self, shopping_list_id: int = 1):
        return self._api_client.clear_shopping_list(shopping_list_id)

    def remove_product_in_shopping_list(
        self, product_id: int, shopping_list_id: int = 1, amount: int = 1
    ):
        return self._api_client.remove_product_in_shopping_list(
            product_id, shopping_list_id, amount
        )

    def product_groups(self) -> List[Group]:
        raw_groups = self._api_client.get_product_groups()
        return [Group(resp) for resp in raw_groups]

    def add_product_pic(self, product_id: int, pic_path: str):
        self._api_client.upload_product_picture(product_id, pic_path)
        return self._api_client.update_product_pic(product_id)

    def get_userfields(self, entity: str, object_id: int):
        return self._api_client.get_userfields(entity, object_id)

    def set_userfields(self, entity: str, object_id: int, key: str, value):
        return self._api_client.set_userfields(entity, object_id, key, value)

    def get_last_db_changed(self):
        return self._api_client.get_last_db_changed()

    def tasks(self) -> List[Task]:
        raw_tasks = self._api_client.get_tasks()
        return [Task(task) for task in raw_tasks]

    def complete_task(self, task_id, done_time):
        return self._api_client.complete_task(task_id, done_time)

    def meal_plan(self, get_details: bool = False) -> List[MealPlanItem]:
        raw_meal_plan = self._api_client.get_meal_plan()
        meal_plan = [MealPlanItem(data) for data in raw_meal_plan]

        if get_details:
            for item in meal_plan:
                item.get_details(self._api_client)
        return meal_plan

    def recipe(self, recipe_id: int) -> RecipeItem:
        recipe = self._api_client.get_recipe(recipe_id)
        if recipe:
            return RecipeItem(recipe)

    def add_generic(self, entity_type, data):
        return self._api_client.add_generic(entity_type, data)

    def batteries(self) -> List[Battery]:
        raw_batteries = self._api_client.get_batteries()
        return [Battery(bat) for bat in raw_batteries]

    def battery(self, battery_id: int) -> Battery:
        battery = self._api_client.get_battery(battery_id)
        if battery:
            return Battery(battery)

    def charge_battery(self, battery_id: int, tracked_time: datetime = datetime.now()):
        return self._api_client.charge_battery(battery_id, tracked_time)



from datetime import datetime
from typing import List

from pygrocy.base import DataModel
from pygrocy.grocy_api_client import (
    CurrentStockResponse,
    GrocyApiClient,
    LocationData,
    MissingProductResponse,
    ProductBarcode,
    ProductDetailsResponse,
    ShoppingListItem,
)


class Product(DataModel):
    def __init__(self, response):
        self._init_empty()
        if isinstance(response, CurrentStockResponse):
            self._init_from_CurrentStockResponse(response)
        elif isinstance(response, MissingProductResponse):
            self._init_from_MissingProductResponse(response)
        elif isinstance(response, ProductDetailsResponse):
            self._init_from_ProductDetailsResponse(response)

    def _init_empty(self):
        self._name = None
        self._id = None
        self._amount_missing = None
        self._is_partly_in_stock = None

        self._available_amount = None
        self._best_before_date = None

        self._barcodes = []
        self._product_group_id = None

    def _init_from_CurrentStockResponse(self, response: CurrentStockResponse):
        self._id = response.product_id
        self._available_amount = response.amount
        self._best_before_date = response.best_before_date
        if response.product:
            self._name = response.product.name
            self._product_group_id = response.product.product_group_id

    def _init_from_MissingProductResponse(self, response: MissingProductResponse):
        self._id = response.product_id
        self._name = response.name
        self._amount_missing = response.amount_missing
        self._is_partly_in_stock = response.is_partly_in_stock

    def _init_from_ProductDetailsResponse(self, response: ProductDetailsResponse):
        self._id = response.product.id
        self._product_group_id = response.product.product_group_id
        self._available_amount = response.stock_amount
        self._best_before_date = response.next_best_before_date
        self._name = response.product.name
        self._barcodes = response.barcodes

    def get_details(self, api_client: GrocyApiClient):
        details = api_client.get_product(self.id)
        if details:
            self._name = details.product.name
            self._barcodes = details.barcodes
            self._product_group_id = details.product.product_group_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def id(self) -> int:
        return self._id

    @property
    def product_group_id(self) -> int:
        return self._product_group_id

    @property
    def available_amount(self) -> float:
        return self._available_amount

    @property
    def best_before_date(self) -> datetime:
        return self._best_before_date

    @property
    def barcodes(self) -> List[str]:
        return [barcode.barcode for barcode in self.product_barcodes]

    @property
    def product_barcodes(self) -> List[ProductBarcode]:
        return self._barcodes

    @property
    def amount_missing(self) -> float:
        return self._amount_missing

    @property
    def is_partly_in_stock(self) -> int:
        return self._is_partly_in_stock


class Group(DataModel):
    def __init__(self, raw_product_group: LocationData):
        self._id = raw_product_group.id
        self._name = raw_product_group.name
        self._description = raw_product_group.description

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description


class ShoppingListProduct(DataModel):
    def __init__(self, raw_shopping_list: ShoppingListItem):
        self._id = raw_shopping_list.id
        self._product_id = raw_shopping_list.product_id
        self._note = raw_shopping_list.note
        self._amount = raw_shopping_list.amount
        self._product = None

    def get_details(self, api_client: GrocyApiClient):
        if self._product_id:
            self._product = Product(api_client.get_product(self._product_id))

    @property
    def id(self) -> int:
        return self._id

    @property
    def product_id(self) -> int:
        return self._product_id

    @property
    def amount(self) -> float:
        return self._amount

    @property
    def note(self) -> str:
        return self._note

    @property
    def product(self) -> Product:
        return self._product



import json
from unittest import TestCase

from pygrocy.grocy_api_client import CurrentStockResponse


class TestCurrentStockResponse(TestCase):
    def test_parse(self):
        input_json = """{"product_id": 0,"amount": "12.53","best_before_date": "2019-04-22","amount_opened": 0, "product": { "id": 0, "name": "test product"}}"""
        response = CurrentStockResponse(json.loads(input_json))

        assert response.product_id == 0
        assert response.amount == 12.53
        assert response.amount_opened == 0
        
        assert response.best_before_date.year == 2019
        assert response.best_before_date.month == 4
        assert response.best_before_date.day == 22
        assert response.product.id == 0
        assert response.product.name == "test product"



import json
from unittest import TestCase

from pygrocy.grocy_api_client import ChoreDetailsResponse


class TestChoreDetailsResponse(TestCase):
    def test_parse(self):
        input_json = """{
            "chore": {
                "id": 0,
                "name": "string",
                "description": "string",
                "period_type": "manually",
                "period_days": 0,
                "row_created_timestamp": "2019-05-04T11:31:04.563Z"
            },
            "last_tracked": "2019-05-04T11:31:04.563Z",
            "track_count": 0,
            "last_done_by": {
                "id": 0,
                "username": "string",
                "first_name": "string",
                "last_name": "string",
                "display_name": "string",
                "row_created_timestamp": "2019-05-04T11:31:04.564Z"
            },
            "next_estimated_execution_time": "2019-05-04T11:31:04.564Z",
            "next_execution_assigned_user": {
                "id": 42,
                "username": "string",
                "first_name": "string",
                "last_name": "string",
                "display_name": "string",
                "row_created_timestamp": "2019-05-04T11:31:04.564Z"
            }
        }"""

        response = ChoreDetailsResponse(json.loads(input_json))

        assert response.chore.id == 0
        assert response.chore.name == "string"

        assert response.last_done_by.display_name == "string"

        assert response.next_execution_assigned_user.id == 42
        assert response.next_execution_assigned_user.display_name == "string"

    def test_no_last_tracked_data(self):
        input_json = """{
            "chore": {
                "id": 0,
                "name": "string",
                "description": "string",
                "period_type": "manually",
                "period_days": 0,
                "row_created_timestamp": "2019-05-04T11:31:04.563Z"
            },
            "last_tracked": null,
            "track_count": 0,
            "last_done_by": null,
            "next_estimated_execution_time": "2019-05-04T11:31:04.564Z"
        }"""

        response = ChoreDetailsResponse(json.loads(input_json))

        assert response.last_tracked is None
        assert response.last_done_by is None



import pytest

from pygrocy.data_models.meal_items import RecipeItem


class TestMealPlan:
    @pytest.mark.vcr
    def test_get_meal_plan_valid(self, grocy):
        meal_plan = grocy.meal_plan(get_details=False)

        assert len(meal_plan) == 8
        item = next(item for item in meal_plan if item.id == 1)
        assert item.day.day == 8
        assert item.recipe_id == 1
        assert item.recipe_servings == 1
        assert item.note is None
        assert item.recipe is None

    @pytest.mark.vcr
    def test_get_meal_plan_with_details_valid(self, grocy):
        meal_plan = grocy.meal_plan(get_details=True)

        assert len(meal_plan) == 8
        item = next(item for item in meal_plan if item.id == 1)
        assert item.day.day == 8
        assert item.recipe_id == 1
        assert item.recipe_servings == 1
        assert item.note is None
        assert isinstance(item.recipe, RecipeItem)

        recipe = item.recipe
        assert recipe.name == "Pizza"
        assert recipe.id == 1
        assert recipe.desired_servings == 1
        assert recipe.base_servings == 1
        assert recipe.description[:4] == "<h1>"
        assert recipe.picture_file_name == "pizza.jpg"



from datetime import datetime
from typing import Dict
from pygrocy.base import DataModel
from pygrocy.grocy_api_client import TaskResponse


class Task(DataModel):
    def __init__(self, response: TaskResponse):

        self._id = response.id
        self._name = response.name
        self._description = response.description
        self._due_date = response.due_date
        self._done = response.done
        self._done_timestamp = response.done_timestamp
        self._category_id = response.category_id
        self._assigned_to_user_id = response.assigned_to_user_id
        self._userfields = response.userfields

    @property
    def id(self) -> str:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def due_date(self) -> datetime:
        return self._due_date

    @property
    def done(self) -> int:
        return self._done

    @property
    def done_timestamp(self) -> datetime:
        return self._done_timestamp

    @property
    def category_id(self) -> int:
        return self._category_id

    @property
    def assigned_to_user_id(self) -> int:
        return self._assigned_to_user_id

    @property
    def userfields(self) -> Dict[str, str]:
        return self._userfields


from datetime import datetime
from enum import Enum
from typing import List, Dict
from pygrocy.base import DataModel
from pygrocy.data_models.user import User
from pygrocy.grocy_api_client import CurrentChoreResponse, ChoreDetailsResponse, GrocyApiClient


class PeriodType(str, Enum):
    MANUALLY = "manually"
    DYNAMIC_REGULAR = "dynamic-regular"
    DAILY = "daily"
    WEEKLY = "weekly"
    MONTHLY = "monthly"
    YEARLY = "yearly"


class AssignmentType(str, Enum):
    NO_ASSIGNMENT = "no-assignment"
    WHO_LEAST_DID_FIRST = "who-least-did-first"
    RANDOM = "random"
    IN_ALPHABETICAL_ORDER = "in-alphabetical-order"


class Chore(DataModel):
    def __init__(self, response):
        if isinstance(response, CurrentChoreResponse):
            self._init_from_CurrentChoreResponse(response)
        elif isinstance(response, ChoreDetailsResponse):
            self._init_from_ChoreDetailsResponse(response)

    # noinspection PyPep8Naming
    def _init_from_CurrentChoreResponse(self, response: CurrentChoreResponse):
        self._id = response.chore_id
        self._last_tracked_time = response.last_tracked_time
        self._next_estimated_execution_time = response.next_estimated_execution_time
        self._name = None
        self._last_done_by = None

    # noinspection PyPep8Naming
    def _init_from_ChoreDetailsResponse(self, response: ChoreDetailsResponse):
        chore_data = response.chore
        self._id = chore_data.id
        self._name = chore_data.name
        self._description = chore_data.description

        if chore_data.period_type is not None:
            self._period_type = PeriodType(chore_data.period_type)
        else:
            self._period_type = None

        self._period_config = chore_data.period_config
        self._period_days = chore_data.period_days
        self._track_date_only = chore_data.track_date_only
        self._rollover = chore_data.rollover

        if chore_data.assignment_type is not None:
            self._assignment_type = AssignmentType(chore_data.assignment_type)
        else:
            self._assignment_type = None

        self._assignment_config = chore_data.assignment_config
        self._next_execution_assigned_to_user_id = (
            chore_data.next_execution_assigned_to_user_id
        )
        self._userfields = chore_data.userfields

        self._last_tracked_time = response.last_tracked
        self._next_estimated_execution_time = response.next_estimated_execution_time
        if response.last_done_by is not None:
            self._last_done_by = User(response.last_done_by)
        else:
            self._last_done_by = None
        self._track_count = response.track_count
        if response.next_execution_assigned_user is not None:
            self._next_execution_assigned_user = User(
                response.next_execution_assigned_user
            )
        else:
            self._next_execution_assigned_user = None

    def get_details(self, api_client: GrocyApiClient):
        details = api_client.get_chore(self.id)
        self._init_from_ChoreDetailsResponse(details)

    @property
    def id(self) -> int:
        return self._id

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def period_type(self) -> PeriodType:
        return self._period_type

    @property
    def period_config(self) -> str:
        return self._period_config

    @property
    def period_days(self) -> int:
        return self._period_days

    @property
    def track_date_only(self) -> bool:
        return self._track_date_only

    @property
    def rollover(self) -> bool:
        return self._rollover

    @property
    def assignment_type(self) -> AssignmentType:
        return self._assignment_type

    @property
    def assignment_config(self) -> str:
        return self._assignment_config

    @property
    def next_execution_assigned_to_user_id(self) -> int:
        return self._next_execution_assigned_to_user_id

    @property
    def userfields(self) -> Dict[str, str]:
        return self._userfields

    @property
    def last_tracked_time(self) -> datetime:
        return self._last_tracked_time

    @property
    def next_estimated_execution_time(self) -> datetime:
        return self._next_estimated_execution_time

    @property
    def last_done_by(self) -> User:
        return self._last_done_by

    @property
    def track_count(self) -> int:
        return self._track_count

    @property
    def next_execution_assigned_user(self) -> User:
        return self._next_execution_assigned_user



import json
from unittest import TestCase

from pygrocy.grocy_api_client import ShoppingListItem


class TestShoppingListItem(TestCase):
    def test_parse(self):
        input_json = """{
            "id": "1",
            "product_id": "6",
            "note": "string",
            "amount": "2",
            "row_created_timestamp": "2019-04-17 10:30:00",
            "shopping_list_id": "1",
            "done": "0"
        }"""

        response = ShoppingListItem(json.loads(input_json))

        assert response.product_id == 6
        assert response.id == 1
        assert response.note == "string"
        assert response.amount == 2
        


import pytest

from pygrocy.data_models.product import Group


class TestProductGroups:
    @pytest.mark.vcr
    def test_get_product_groups_valid(self, grocy):
        product_groups_list = grocy.product_groups()

        assert isinstance(product_groups_list, list)
        assert len(product_groups_list) == 6
        for group in product_groups_list:
            assert isinstance(group, Group)
            assert isinstance(group.id, int)
            assert isinstance(group.name, str)
            if group.description:
                assert isinstance(group.description, str)

        group = next(group for group in product_groups_list if group.id == 1)
        assert group.name == "01 Sweets"
        assert group.description is None



import json
from unittest import TestCase

from pygrocy.grocy_api_client import UserDto


class TestUserDto(TestCase):
    def test_parse(self):
        input_json = """{
            "id": "1",
            "username": "user",
            "first_name": "Guz",
            "last_name": "Userman",
            "row_created_timestamp": "2019-04-17 10:30:00",
            "display_name": "Guzzboy"
        }"""

        response = UserDto(json.loads(input_json))

        assert response.display_name == "Guzzboy"
        assert response.id == 1
        assert response.first_name == "Guz"
        assert response.last_name == "Userman"
        assert response.username == "user"



from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL

import pytest

from pygrocy import Grocy


@pytest.fixture
def grocy():
    yield Grocy(CONST_BASE_URL, "demo_mode", verify_ssl=CONST_SSL, port=CONST_PORT)


# noinspection PyProtectedMember
@pytest.fixture
def grocy_api_client(grocy):
    yield grocy._api_client


@pytest.fixture
def vcr_config():
    return {"record_mode": "once"}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pygrocy",
    version="0.27.0",
    author="Sebastian Rutofski",
    author_email="kontakt@sebastian-rutofski.de",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sebrut/pygrocy",
    license="MIT",
    packages=setuptools.find_packages(),
    install_requires=["requests", "iso8601", "pytz", "tzlocal", "deprecation"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import pytest


class TestProduct:
    @pytest.mark.vcr
    def test_product_get_details_valid(self, grocy):
        product = grocy.product(10)

        assert product.name == "Cheese"
        assert product.available_amount == 5
        assert len(product.barcodes) == 0
        assert product.product_group_id == 6

    @pytest.mark.vcr
    def test_product_no_barcodes(self, grocy):
        stock = grocy.stock()
        product = next(prod for prod in stock if prod.id == 2)

        assert product.name == "Chocolate"
        assert product.product_barcodes is not None
        assert isinstance(product.product_barcodes, list)
        assert len(product.product_barcodes) == 0

    @pytest.mark.vcr
    def test_product_get_details_non_existant(self, grocy):
        import requests

        with pytest.raises(requests.exceptions.HTTPError):
            grocy.product(200)

    @pytest.mark.vcr
    def test_add_product_pic_valid(self, grocy, mocker):
        mocked_exists = mocker.patch("os.path.exists")
        mocked_exists.return_value = True

        mocker.patch("builtins.open", mocker.mock_open())

        assert grocy.add_product_pic(20, "/somepath/pic.jpg") is None



from datetime import datetime

import pytest


class TestSystem:
    @pytest.mark.vcr
    def test_get_last_db_changed_valid(self, grocy):
        timestamp = grocy.get_last_db_changed()

        assert isinstance(timestamp, datetime)
        assert timestamp.year == 2021
        assert timestamp.month == 2
        assert timestamp.day == 13



import pytest

from pygrocy.data_models.product import Product, ShoppingListProduct


class TestShoppingList:
    @pytest.mark.vcr
    def test_get_shopping_list_valid(self, grocy):
        shopping_list = grocy.shopping_list(True)

        assert isinstance(shopping_list, list)
        assert len(shopping_list) == 7
        for item in shopping_list:
            assert isinstance(item, ShoppingListProduct)
            assert isinstance(item.id, int)
            if item.product_id:
                assert isinstance(item.product_id, int)
                assert isinstance(item.product, Product)
                assert isinstance(item.product.id, int)
            assert isinstance(item.amount, float)
            if item.note:
                assert isinstance(item.note, str)

        item = next(item for item in shopping_list if item.id == 2)
        assert item.note is None
        assert item.amount == 1.0
        assert item.product_id == 20

    @pytest.mark.vcr
    def test_add_missing_product_to_shopping_list_valid(self, grocy):
        assert grocy.add_missing_product_to_shopping_list() is None

    @pytest.mark.vcr
    def test_add_product_to_shopping_list_valid(self, grocy):
        grocy.add_product_to_shopping_list(19)

    @pytest.mark.vcr
    def test_add_product_to_shopping_list_error(self, grocy):
        import requests

        with pytest.raises(requests.exceptions.HTTPError):
            grocy.add_product_to_shopping_list(3000)

    @pytest.mark.vcr
    def test_clear_shopping_list_valid(self, grocy):
        grocy.clear_shopping_list()

    @pytest.mark.vcr
    def test_remove_product_in_shopping_list_valid(self, grocy):
        grocy.remove_product_in_shopping_list(20)



import json
from unittest import TestCase

from pygrocy.grocy_api_client import LocationData


class TestLocationData(TestCase):
    def test_parse(self):
        input_json = """{
            "id": "1",
            "name": "string",
            "description": "string",
            "row_created_timestamp": "2019-04-17 10:30:00"
        }"""

        response = LocationData(json.loads(input_json))

        assert response.id == 1
        assert response.name == "string"
        assert response.description == "string"
        


import json
from datetime import datetime
from unittest import TestCase

from pygrocy.grocy_api_client import MealPlanResponse


class TestMealPlanResponse(TestCase):
    def test_parse(self):
        input_json = """{
            "id": "7",
            "day": "2020-08-16",
            "type": "recipe",
            "recipe_id": "4",
            "recipe_servings": "3",
            "note": null,
            "product_id": null,
            "product_amount": "0.0",
            "product_qu_id": null,
            "row_created_timestamp": "2020-08-12 14:37:06",
            "userfields": null
        }"""

        response = MealPlanResponse(json.loads(input_json))

        assert response.id == 7
        assert response.recipe_id == 4
        assert response.recipe_servings == 3
        self.assertIsInstance(response.day, datetime)



from pygrocy.base import DataModel
from pygrocy.grocy_api_client import UserDto


class User(DataModel):
    def __init__(self, user_dto: UserDto):
        self._id = user_dto.id
        self._username = user_dto.username
        self._first_name = user_dto.first_name
        self._last_name = user_dto.last_name
        self._display_name = user_dto.display_name

    @property
    def id(self) -> int:
        return self._id

    @property
    def username(self) -> str:
        return self._username

    @property
    def first_name(self) -> str:
        return self._first_name

    @property
    def last_name(self) -> str:
        return self._last_name

    @property
    def display_name(self) -> str:
        return self._display_name